Package: ChainLadder
Type: Package
Title: Statistical Methods and Models for Claims Reserving in General
        Insurance
Version: 0.2.21
Date: 2026-02-18
Authors@R: c(person("Markus", "Gesmann", role = c("aut", "cre"),
             email = "markus.gesmann@googlemail.com"),
             person("Daniel", "Murphy", role = "aut", 
             email="danielmarkmurphy@gmail.com"),
             person("Yanwei (Wayne)", "Zhang", role = "aut", 
             email="actuary_zhang@hotmail.com"),
             person("Alessandro", "Carrato", role = "aut", 
             email="alessandro.carrato@gmail.com"),
             person("Giuseppe", "Crupi", role="ctb",
             email="crupi.giuseppe@gmail.com"),
             person("Christophe", "Dutang", role="ctb"),
             person("Arnaud", "Lacoume", role="ctb"),               
             person("Arthur", "Charpentier", role = "ctb"),
             person("Mario", "Wuthrich", role = "aut",
             email = "mario.wuethrich@math.ethz.ch"),
             person("Fabio", "Concina", role="aut",
             email="fabio.concina@gmail.com"),
             person("Eric", "Dal Moro", role="aut", 
             email="eric_dal_moro@yahoo.com"),
             person("Yuriy", "Krvavych", role="ctb", 
             email="y.krvavych@gmail.com"),
             person("Vincent", "Goulet", role="ctb",
             email="vincent.goulet@act.ulaval.ca"),
             person("Marco", "De Virgilis",  role="ctb",
             email="devirgilis.marco@gmail.com"),
             person("Marco", "Spina",  role="ctb")
             )  
Description: Various statistical methods and models which are
    typically used for the estimation of outstanding claims reserves
    in general insurance, including those to estimate the claims
    development result as required under Solvency II.
Imports: Matrix, actuar, methods, stats, lattice, grid, tweedie, utils,
        systemfit, statmod, cplm (>= 0.7-3), ggplot2, MASS
Suggests: RUnit, knitr, rmarkdown
VignetteBuilder: knitr
License: GPL (>= 2)
URL: https://mages.github.io/ChainLadder/
BugReports: https://github.com/mages/ChainLadder/issues
LazyLoad: yes
LazyData: yes
RoxygenNote: 7.1.1
NeedsCompilation: no
Packaged: 2026-02-18 07:37:57 UTC; mages
Author: Markus Gesmann [aut, cre],
  Daniel Murphy [aut],
  Yanwei (Wayne) Zhang [aut],
  Alessandro Carrato [aut],
  Giuseppe Crupi [ctb],
  Christophe Dutang [ctb],
  Arnaud Lacoume [ctb],
  Arthur Charpentier [ctb],
  Mario Wuthrich [aut],
  Fabio Concina [aut],
  Eric Dal Moro [aut],
  Yuriy Krvavych [ctb],
  Vincent Goulet [ctb],
  Marco De Virgilis [ctb],
  Marco Spina [ctb]
Maintainer: Markus Gesmann <markus.gesmann@googlemail.com>
Repository: CRAN
Date/Publication: 2026-02-18 08:30:02 UTC
Built: R 4.6.0; ; 2026-05-01 01:19:26 UTC; unix
